#!/usr/bin/perl
#
# This script updates a list of Nortel 350-24T 
# switches from one version of microcode to another.
# by Anthony Anderberg - 2.7.2002
#
# http://www.anderbergfamily.net/ant/autoupdate/
#

#############################################
#
# Other people's code
#

use Net::SNMP;

#############################################
#
# Static variables
#

# Our version number
$version = '1.1 - 7.19.2002';

# Our configuration file
$conf_filename = 'autoupdate_conf.txt';


#############################################
#
# These are our important OIDs
#

# We'll get and print this for informational purposes
$sysDescr_oid = '.1.3.6.1.2.1.1.1.0';

# We'll check to make sure it's the right kind of device.
$sysObjectID_oid = '.1.3.6.1.2.1.1.2.0';

# This holds the current boot code version
$boot_oid = '.1.3.6.1.4.1.45.1.6.3.5.1.1.7.3.1.0.2';

# This holds the current agent code version
# .1.3.6.1.4.1.45.1.6.3.5.1.1.7.3.1.0.2 also might work.
$agent_oid = '.1.3.6.1.4.1.45.1.6.4.2.1.10.0';

# This is set to the TFTP server's IP address
# Type: IPADDRESS
# enterprises.synoptics.products.series5000.s5Agent.s5AgentInfo.s5AgMyIfTable.s5AgMyIfEntry.s5AgMyIfLdSvrAddr.1
$tftpserver_oid = '.1.3.6.1.4.1.45.1.6.4.2.2.1.5.1';

# This is the image file to be gotten from TFTP
# Type: OCTET_STRING
# enterprises.synoptics.products.series5000.s5Agent.s5AgentInfo.s5AgMyIfTable.s5AgMyIfEntry.s5AgMyIfImgFname.1
$tftpfilename_oid = '.1.3.6.1.4.1.45.1.6.4.2.2.1.6.1';

# This tells the switch to start the data transfer,
# is set to 3 for the software image.
# Type: INTEGER
# enterprises.synoptics.products.series5000.s5Agent.s5AgentInfo.s5AgentGbl.s5AgInfoFileAction.0
$tftpstartcommand_oid = '.1.3.6.1.4.1.45.1.6.4.2.1.24.0';

# status of the latest action as shown by s5AgInfoFileAction
#    other(1) ......... if no action taken since the boot up
#    inProgress (2) ... the operation is in progress
#    success (3) ...... the operation succeeds.
#    fail (4) ......... the operation is failed. 
# Type: INTEGER
# enterprises.synoptics.products.series5000.s5Agent.s5AgentInfo.s5AgentGbl.s5AgInfoFileStatus.0
$tftpcommandstatus_oid = '.1.3.6.1.4.1.45.1.6.4.2.1.25.0';


#############################################
#
# Setup work...
#

#
# Process our configuration file
# This should be formatted like this:
# variable=value\n
#
open(CONF, "$conf_filename");
while (<CONF>)
 {
  chomp ($_);
  ($conf_variable, $conf_value) = split(/=/, $_, 2);
  $config{$conf_variable} = $conf_value;
 }
close (CONF);

#
# Open our log file
#
open (LOG, ">$config{log_filename}");

#
# Print our program settings
#
report("\nAutoupdate version $version\n\nConfiguration:\n");
while (($f, $l) = each(%config))
 {
  report("    $f = $l\n");
 }

#
# Read in our list of devices.
#
open(DEVICES, "$config{device_filename}");
@devicelist = <DEVICES>;
close(DEVICES);

#############################################
#
# Make sure the nice user has started their TFTP server
#
print "\nMake sure your TFTP server is active, then press Enter to continue.";
$foo = <STDIN>;

#############################################
#
# Look at each device, upgrade them one at a time.
# If anything seems weird skip the device, 
# and if anything goes wrong abort completely.
#

foreach $device (@devicelist)
  {
   chomp($device);  # remove the carrage return
   report("\nDevice: $device\n");

   # Setup our SNMP session
   ($snmp_session, $snmp_error) = Net::SNMP->session(HostName => $device, Community => $config{community});

   #####################
   # First we get some basic data and check that its the right kind of device.
   #

   # Get the sysObjectID
   $result = $snmp_session->get_request("$sysObjectID_oid");
   $device_sysobjectid = $result->{$sysObjectID_oid};

   # Get the system description
   $result = $snmp_session->get_request("$sysDescr_oid");
   $device_sysdescr = $result->{$sysDescr_oid};

   # Print out the current settings and check to make sure they are what we're expecting.
   report("  Description: $device_sysdescr\n");
   report("  sysObjectID: $device_sysobjectid\n");
   if ($device_sysobjectid ne $config{system_objectid})
     {
      report("-> Error: expected sysObjectID $config{system_objectid} skipping device!\n");
      next;
     }

   #####################
   # Next we make sure that the current microcode version numbers are what we expect.
   #

   # Get the current Boot code version
   $result = $snmp_session->get_request("$boot_oid");
   $device_oldboot = $result->{$boot_oid};

   # Get the current Agent code version
   $result = $snmp_session->get_request("$agent_oid");
   $device_oldagent = $result->{$agent_oid};

   # Remove any spaces which sometimes show up after version numbers
   $device_oldboot =~ s/ //;
   $device_oldagent =~ s/ //;
   # Sometimes these variables have V or v in front, removed them.
   $device_oldboot =~ s/V//;
   $device_oldboot =~ s/v//;
   $device_oldagent =~ s/V//;
   $device_oldagent =~ s/v//;

   report("  Current Boot Version: $device_oldboot\n");
   if ($device_oldboot ne $config{old_boot_version})
     {
      report("-> Error: expected $config{old_boot_version} skipping device!\n");
      next;
     }
   report("  Current Agent Version: $device_oldagent\n");
   if ($device_oldagent ne $config{old_agent_version})
     {
      report("-> Error: expected $config{old_agent_version} skipping device!\n");
      next;
     }

   #####################
   # Now we upgrade the boot code.
   #
   report("  Beginning Boot Code Upgrade Process:\n");

   # Set the TFTP server address
   report("    Setting TFTP address.");
   $result = $snmp_session->set_request($tftpserver_oid, IPADDRESS, $config{'tftp_server_ip'});
   report(" Sent:$config{'tftp_server_ip'}  Rcvd:$result->{$tftpserver_oid} - ");
   if ($result->{$tftpserver_oid} eq $config{'tftp_server_ip'})
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Error: TFTP Server Address could not be set, skipping device!\n");
      next;
     }

   # Set the TFTP file name
   report("    Setting TFTP file name.");
   $result = $snmp_session->set_request($tftpfilename_oid, OCTET_STRING, $config{'new_boot_filename'});
   report(" Sent:$config{'new_boot_filename'}  Rcvd:$result->{$tftpfilename_oid} - ");
   if ($result->{$tftpfilename_oid} eq $config{'new_boot_filename'})
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Error: TFTP File Name could not be set, skipping device!\n");
      next;
     }

   #
   # If there are any errors from here on we should completely abort.
   #

   # Set the s5AgInfoFileAction to '3' for file download to the device
   report("    Sending command to start transfer.");
   $result = $snmp_session->set_request($tftpstartcommand_oid, INTEGER, 3);
   report(" Sent:3  Rcvd:$result->{$tftpstartcommand_oid} - ");
   if ($result->{$tftpstartcommand_oid} eq '3')
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Unexpected failure, program aborting.\n\n");
      exit;
     }

   # wait update_pause seconds while the upgrade and reboot happen
   report("    Pausing $config{'update_pause'} seconds for transfer and reboot: ");
   sleep($config{'update_pause'});
   report("done.\n");

   # check that the version has changed.
   report("    Checking new boot code version number: ");
   $result = $snmp_session->get_request("$boot_oid");
   $result->{$boot_oid} =~ s/ //;
   $result->{$boot_oid} =~ s/V//;
   $result->{$boot_oid} =~ s/v//;
   if ($result->{$boot_oid} eq $config{'new_boot_version'})
     {
      report("$result->{$boot_oid} - OK.\n");
     }
   else    
     {
      report("$result->{$boot_oid} - failed!\n-> Unexpected failure, program aborting.\n\n");
      exit;
     }


   #####################
   # So far so good, now we upgrade the agent code.
   #

   report("  Beginning Agent Code Upgrade Process:\n");

   # Set the TFTP server address
   report("    Setting TFTP address.");
   $result = $snmp_session->set_request($tftpserver_oid, IPADDRESS, $config{'tftp_server_ip'});
   report(" Sent:$config{'tftp_server_ip'}  Rcvd:$result->{$tftpserver_oid} - ");
   if ($result->{$tftpserver_oid} eq $config{'tftp_server_ip'})
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Error: TFTP Server Address could not be set, skipping device!\n");
      next;
     }

   # Set the TFTP file name
   report("    Setting TFTP file name: ");
   $result = $snmp_session->set_request($tftpfilename_oid, OCTET_STRING, $config{'new_agent_filename'});
   report(" Sent:$config{'new_agent_filename'}  Rcvd:$result->{$tftpfilename_oid} - ");
   if ($result->{$tftpfilename_oid} eq $config{'new_agent_filename'})
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Error: TFTP file name could not be set, skipping device!\n");
      next;
     }

   # Set the s5AgInfoFileAction to '3' for file download to the device
   report("    Sending command to start transfer: ");
   $result = $snmp_session->set_request($tftpstartcommand_oid, INTEGER, 3);
   report(" Sent:3  Rcvd:$result->{$tftpstartcommand_oid} - ");
   if ($result->{$tftpstartcommand_oid} eq '3')
     {
      report("OK.\n");
     }
   else
     {
      report("failed!\n-> Unexpected failure, program aborting.\n\n");
      exit;
     }

   # wait update_pause seconds while the upgrade and reboot happen
   report("    Pausing $config{'update_pause'} seconds for transfer and reboot: ");
   sleep($config{'update_pause'});
   report("done.\n");

   # check that the version has changed.
   report("    Checking new agent code version number: ");
   $result = $snmp_session->get_request("$agent_oid");
   $result->{$agent_oid} =~ s/ //;
   $result->{$agent_oid} =~ s/V//;
   $result->{$agent_oid} =~ s/v//;
   if ($result->{$agent_oid} eq $config{'new_agent_version'})
     {
      report("$result->{$agent_oid} - OK.\n");
     }
   else    
     {
      report("$result->{$agent_oid} - failed!\n-> Unexpected failure, program aborting.\n\n");
      exit;
     }


   #####################
   # Everything looks good, declare victory and move on...
   #

   # Get the system description
   $result = $snmp_session->get_request("$sysDescr_oid");
   $device_sysdescr = $result->{$sysDescr_oid};

   # Get the current Boot code version
   $result = $snmp_session->get_request("$boot_oid");
   $device_boot = $result->{$boot_oid};

   # Get the current Agent code version
   $result = $snmp_session->get_request("$agent_oid");
   $device_agent = $result->{$agent_oid};

   # Remove any spaces which sometimes show up after version numbers
   $device_boot =~ s/ //;
   $device_agent =~ s/ //;
   # Sometimes these variables have V or v in front, removed them.
   $device_boot =~ s/V//;
   $device_boot =~ s/v//;
   $device_agent =~ s/V//;
   $device_agent =~ s/v//;

   report("  New Boot Version: $device_boot\n");
   report("  New Agent Version: $device_agent\n");
   report("  Description: $device_sysdescr\n");

   $snmp_session->close;
   report("  Upgrade Process is Complete.\n");

  } # The end of our main loop


#
# The end.
#
close (LOG);

#############################################
#
# This prints a given value to the screen and our log file.
#
sub report()
 {
  print $_[0];
  print LOG $_[0];
 }
